index_tokens = {
    'NIFTY 50': 26000,
    'NIFTY BANK': 26009,
    'NIFTY NEXT 50': 26013,
    'NIFTY FIN SERVICE': 26037,
    'NIFTY FINANCIAL SERVICES': 26037,
    'NIFTY MIDCAP 100': 26010,    
    'NIFTY MID SELECT': 26074,
    'NIFTY MIDCAP SELECT': 26074,
    'NIFTY TOTAL MKT': 26021,
    'NIFTY TOTAL MARKET': 26021,
    'NIFTY IT': 26008,
    'NIFTY 500': 26003,
    'NIFTY 100': 26012,
    'INDIA VIX': 26017,
    'NIFTY MIDCAP 50': 26014,
    'NIFTY REALTY': 26052,
    'NIFTY INFRA': 26019,
    'NIFTY ENERGY': 26054,
    'NIFTY FMCG': 26055,
    'NIFTY MNC': 26056,
    'NIFTY PHARMA': 26057,
    'NIFTY PSE': 26024,
    'NIFTY PSU BANK': 26059,
    'NIFTY SERV SECTOR': 26060,
    'NIFTY SMLCAP 100': 26011,
    'NIFTY SMALLCAP 100': 26011,
    'NIFTY 200': 26065,
    'NIFTY AUTO': 26061,
    'NIFTY MEDIA': 26063,
    'NIFTY METAL': 26062,
    'NIFTY DIV OPPS 50': 26078,
    'NIFTY COMMODITIES': 26066,
    'NIFTY CONSUMPTION': 26048,
    'NIFTY50 DIV POINT': 26080,
    'NIFTY100 LIQ 15': 26051,
    'NIFTY CPSE': 26041,
    'NIFTY GROWSECT 15': 26082,
    'NIFTY50 TR 2X LEV': 26075,
    'NIFTY50 PR 2X LEV': 26064,
    'NIFTY50 TR 1X INV': 26076,
    'NIFTY50 PR 1X INV': 26020,
    'NIFTY50 VALUE 20': 26081,
    'NIFTY100 QUALTY30': 26026,
    'NIFTY MID LIQ 15': 26018,
    'NIFTY PVT BANK': 26025,
    'NIFTY PRIVATE BANK': 26025,
    'NIFTY GS 8 13YR': 26053,
    'NIFTY GS 10YR': 26058,
    'NIFTY GS 10YR CLN': 26029,
    'NIFTY GS 4 8YR': 26030,
    'NIFTY GS 11 15YR': 26069,
    'NIFTY GS 15YRPLUS': 26070,
    'NIFTY GS COMPSITE': 26071,
    'NIFTY50 EQL WGT': 26083,
    'NIFTY100 EQL WGT': 26084,
    'NIFTY100 LOWVOL30': 26085,
    'NIFTY ALPHA 50': 26086,
    'NIFTY MIDCAP 150': 26087,
    'NIFTY SMLCAP 50': 26088,
    'NIFTY SMALLCAP 50': 26088,
    'NIFTY SMLCAP 250': 26089,
    'NIFTY SMALLCAP 250': 26089,
    'NIFTY MIDSML 400': 26091,
    'NIFTY200 QUALTY30': 26092,
    'NIFTY FINSRV25 50': 26093,
    'NIFTY ALPHALOWVOL': 26094,
    'NIFTY200MOMENTM30': 26067,
    'NIFTY100ESGSECLDR': 26068,
    'NIFTY HEALTHCARE': 26096,
    'NIFTY HEALTHCARE INDEX': 26096,
    'NIFTY CONSR DURBL': 26095,
    'NIFTY CONSUMER DURABLES': 26095,
    'NIFTY OIL AND GAS': 26099,
    'NIFTY OIL & GAS' : 26099,
    'NIFTY500 MULTICAP': 26098,
    'NIFTY LARGEMID250': 26097,
    'NIFTY MICROCAP250': 26022,
    'NIFTY MICROCAP 250': 26022,
    'NIFTY IND DIGITAL': 26023,
    'NIFTY100 ESG': 26027,
    'NIFTY M150 QLTY50': 26028,
    'NIFTY INDIA MFG': 26031,
    'NIFTY200 ALPHA 30': 26043,
    'NIFTYM150MOMNTM50': 26042,
    'NIFTY TATA 25 CAP': 26047,
    'NIFTY MIDSML HLTH': 26044,
    'NIFTY MIDSMALL HEALTHCARE' : 26044,
    'NIFTY MULTI MFG': 26045,
    'NIFTY MULTI INFRA': 26046,
    'NIFTY IND DEFENCE': 26038,
    'NIFTY IND TOURISM': 26039,
    'NIFTY CAPITAL MKT': 26040,
    'NIFTY500MOMENTM50': 26049,
    'NIFTYMS400 MQ 100': 26050,
    'NIFTYSML250MQ 100': 26079,
    'NIFTY TOP 10 EW': 26077,
    'NIFTY AQL 30': 26100,
    'NIFTY AQLV 30': 26101,
    'NIFTY EV': 26102,
    'NIFTY HIGHBETA 50': 26103,
    'NIFTY NEW CONSUMP': 26104,
    'NIFTY CORP MAATR': 26105,
    'NIFTY LOW VOL 50': 26106,
    'NIFTY MOBILITY': 26107,
    'NIFTY QLTY LV 30': 26108,
    'NIFTY SML250 Q50': 26109,
    'NIFTY TOP 15 EW': 26110,
    'NIFTY100 ALPHA 30': 26111,
    'NIFTY100 ENH ESG': 26112,
    'NIFTY200 VALUE 30': 26113,
    'NIFTY500 EW': 26114,
    'NIFTY MULTI MQ 50': 26115,
    'NIFTY500 VALUE 50': 26116,
    'NIFTY TOP 20 EW': 26117,
    'NIFTY COREHOUSING': 26118,
    'NIFTY FINSEREXBNK': 26119,
    'NIFTY HOUSING': 26120,
    'NIFTY IPO': 26121,
    'NIFTY MS FIN SERV': 26122,
    'NIFTY MIDSMALL FINANCIAL SERVICES': 26122,
    'NIFTY MS IND CONS': 26123,
    'NIFTY MS IT TELCM': 26124,
    'NIFTY MIDSMALL IT & TELECOM' : 26124,
    'NIFTY NONCYC CONS': 26125,
    'NIFTY RURAL': 26126,
    'NIFTY SHARIAH 25': 26127,
    'NIFTY TRANS LOGIS': 26128,
    'NIFTY50 SHARIAH': 26129,
    'NIFTY500 LMS EQL': 26130,
    'NIFTY500 SHARIAH': 26131,
    'NIFTY500 QLTY50': 26132,
    'NIFTY500 LOWVOL50': 26133,
    'NIFTY500 MQVLV50': 26134,
}